@@ -3,22 +3,26 @@ import unittest
 
 from six import PY3
 
+from AccessControl import safe_builtins
 from Products.PageTemplates.PageTemplateFile import PageTemplateFile
 from Testing.ZopeTestCase import ZopeTestCase
 
+from .util import useChameleonEngine
+
 
 path = os.path.dirname(__file__)
 
 
 class TestPageTemplateFile(ZopeTestCase):
+
     def afterSetUp(self):
-        from Zope2.App import zcml
-        import Products.PageTemplates
-        zcml.load_config("configure.zcml", Products.PageTemplates)
+        useChameleonEngine()
 
     def _makeOne(self, name):
         return PageTemplateFile(os.path.join(path, name)).__of__(self.app)
 
+    @unittest.skipIf("sorted" not in safe_builtins,
+                     "`sorted` not allowed by `AccessControl`")
     def test_rr(self):
         class Prioritzed(object):
             __allow_access_to_unprotected_subobjects__ = 1
