@@ -4,11 +4,10 @@ These tests mainly verify that OFS.Traversable.absolute_url()
 works correctly in a VHM environment.
 
 Also see http://zope.org/Collectors/Zope/809
-
-Note: Tests require Zope >= 2.7
 """
 import unittest
 
+
 class VHMRegressions(unittest.TestCase):
 
     def setUp(self):
@@ -17,7 +16,7 @@ class VHMRegressions(unittest.TestCase):
         from Testing.ZopeTestCase.ZopeLite import app
         transaction.begin()
         self.app = makerequest(app())
-        if 'virtual_hosting' not in  self.app.objectIds():
+        if 'virtual_hosting' not in self.app.objectIds():
             # If ZopeLite was imported, we have no default virtual
             # host monster
             from Products.SiteAccess.VirtualHostMonster \
@@ -35,7 +34,7 @@ class VHMRegressions(unittest.TestCase):
 
     def testAbsoluteUrl(self):
         m = self.app.folder.doc.absolute_url
-        self.assertEqual(m(), 'http://foo/folder/doc')
+        self.assertEqual(m(), 'http://nohost/folder/doc')
 
     def testAbsoluteUrlPath(self):
         m = self.app.folder.doc.absolute_url_path
@@ -52,64 +51,85 @@ class VHMRegressions(unittest.TestCase):
         self.assertEqual(m(), ('', 'folder', 'doc'))
 
     def test_actual_url_no_VHR_no_doc_w_trailing_slash(self):
-        ob = self.traverse('/VirtualHostBase/http/www.mysite.com:80'
-                           '/folder/')
+        self.traverse('/VirtualHostBase/http/www.mysite.com:80'
+                      '/folder/')
         self.assertEqual(self.app.REQUEST['ACTUAL_URL'],
-                        'http://www.mysite.com/folder/')
+                         'http://www.mysite.com/folder/')
 
     def test_actual_url_no_VHR_no_doc_no_trailing_slash(self):
-        ob = self.traverse('/VirtualHostBase/http/www.mysite.com:80'
-                           '/folder')
+        self.traverse('/VirtualHostBase/http/www.mysite.com:80'
+                      '/folder')
         self.assertEqual(self.app.REQUEST['ACTUAL_URL'],
                          'http://www.mysite.com/folder')
 
     def test_actual_url_no_VHR_w_doc_w_trailing_slash(self):
-        ob = self.traverse('/VirtualHostBase/http/www.mysite.com:80'
-                           '/folder/doc/')
+        self.traverse('/VirtualHostBase/http/www.mysite.com:80'
+                      '/folder/doc/')
         self.assertEqual(self.app.REQUEST['ACTUAL_URL'],
-                        'http://www.mysite.com/folder/doc/')
+                         'http://www.mysite.com/folder/doc/')
 
     def test_actual_url_no_VHR_w_doc_no_trailing_slash(self):
-        ob = self.traverse('/VirtualHostBase/http/www.mysite.com:80'
-                           '/folder/doc')
+        self.traverse('/VirtualHostBase/http/www.mysite.com:80'
+                      '/folder/doc')
         self.assertEqual(self.app.REQUEST['ACTUAL_URL'],
                          'http://www.mysite.com/folder/doc')
 
     def test_actual_url_w_VHR_w_doc_w_trailing_slash(self):
-        ob = self.traverse('/VirtualHostBase/http/www.mysite.com:80'
-                           '/folder/VirtualHostRoot/doc/')
+        self.traverse('/VirtualHostBase/http/www.mysite.com:80'
+                      '/folder/VirtualHostRoot/doc/')
         self.assertEqual(self.app.REQUEST['ACTUAL_URL'],
-                        'http://www.mysite.com/doc/')
+                         'http://www.mysite.com/doc/')
 
     def test_actual_url_w_VHR_w_doc_no_trailing_slash(self):
-        ob = self.traverse('/VirtualHostBase/http/www.mysite.com:80'
-                           '/folder/VirtualHostRoot/doc')
+        self.traverse('/VirtualHostBase/http/www.mysite.com:80'
+                      '/folder/VirtualHostRoot/doc')
         self.assertEqual(self.app.REQUEST['ACTUAL_URL'],
                          'http://www.mysite.com/doc')
 
     def test_actual_url_w_VHR_no_doc_w_trailing_slash(self):
-        ob = self.traverse('/VirtualHostBase/http/www.mysite.com:80'
-                           '/folder/VirtualHostRoot/')
+        self.traverse('/VirtualHostBase/http/www.mysite.com:80'
+                      '/folder/VirtualHostRoot/')
         self.assertEqual(self.app.REQUEST['ACTUAL_URL'],
                          'http://www.mysite.com/')
 
     def test_actual_url_w_VHR_no_doc_no_trailing_slash(self):
-        ob = self.traverse('/VirtualHostBase/http/www.mysite.com:80'
-                           '/folder/VirtualHostRoot')
+        self.traverse('/VirtualHostBase/http/www.mysite.com:80'
+                      '/folder/VirtualHostRoot')
         self.assertEqual(self.app.REQUEST['ACTUAL_URL'],
                          'http://www.mysite.com/')
 
+    def testWildcardRewrite(self):
+        ob = self.traverse('/VirtualHostBase/http/doc.example.com:80'
+                           '/folder/*/VirtualHostRoot')
+        self.assertEqual(self.app.REQUEST['ACTUAL_URL'],
+                         'http://doc.example.com/')
+        self.assertEqual(ob.getPhysicalPath(), ('', 'folder', 'doc'))
+
+    def test_vhm_set_map(self):
+        vhm = self.app.virtual_hosting
+        mapping = """
+somedomain1/folder
+
+
+somedomain2/folder
+        """  # map with empty lines.
+        vhm.set_map(mapping)
+
+        self.assertEqual(len(vhm.lines), 2)
+        self.assertEqual(vhm.lines[0], 'somedomain1/folder')
+        self.assertEqual(vhm.lines[1], 'somedomain2/folder')
+
+
 def gen_cases():
     for vbase, ubase in (
-        ('', 'http://foo'),
-        ('/VirtualHostBase/http/example.com:80', 'http://example.com'),
-        ):
+            ('', 'http://nohost'),
+            ('/VirtualHostBase/http/example.com:80', 'http://example.com')):
         yield vbase, '', '', 'folder/doc', ubase
+
         for vr, _vh, p in (
-            ('folder', '', 'doc'),
-            ('folder', 'foo', 'doc'),
-            ('', 'foo', 'folder/doc'),
-            ):
+                ('folder', '', 'doc'),
+                ('folder', 'foo', 'doc'),
+                ('', 'foo', 'folder/doc')):
             vparts = [vbase, vr, 'VirtualHostRoot']
             if not vr:
                 del vparts[1]
@@ -117,6 +137,7 @@ def gen_cases():
                 vparts.append('_vh_' + _vh)
             yield '/'.join(vparts), vr, _vh, p, ubase
 
+
 for i, (vaddr, vr, _vh, p, ubase) in enumerate(gen_cases()):
     def test(self, vaddr=vaddr, vr=vr, _vh=_vh, p=p, ubase=ubase):
         ob = self.traverse('%s/%s/' % (vaddr, p))
@@ -139,6 +160,7 @@ for i, (vaddr, vr, _vh, p, ubase) in enumerate(gen_cases()):
 
     setattr(VHMRegressions, 'testTraverse%s' % i, test)
 
+
 class VHMPort(unittest.TestCase):
 
     def setUp(self):
@@ -147,7 +169,7 @@ class VHMPort(unittest.TestCase):
         from Testing.ZopeTestCase.ZopeLite import app
         transaction.begin()
         self.app = makerequest(app())
-        if 'virtual_hosting' not in  self.app.objectIds():
+        if 'virtual_hosting' not in self.app.objectIds():
             # If ZopeLite was imported, we have no default virtual
             # host monster
             from Products.SiteAccess.VirtualHostMonster \
@@ -229,24 +251,18 @@ class VHMAddingTests(unittest.TestCase):
 
         vhm2 = self._makeOne()
         self.assertRaises(BadRequest, vhm2.manage_addToContainer, self.root)
-        self.assertRaises( BadRequest
-                         , manage_addVirtualHostMonster
-                         , self.root
-                         )
+        self.assertRaises(BadRequest, manage_addVirtualHostMonster, self.root)
 
     def test_add_id_collision(self):
         from OFS.Folder import Folder
         from Products.SiteAccess.VirtualHostMonster import \
             manage_addVirtualHostMonster
         from zExceptions import BadRequest
-        self.root._setObject('VHM', Folder('VHM'))
+        self.root._setObject('virtual_hosting', Folder('virtual_hosting'))
         vhm1 = self._makeOne()
 
         self.assertRaises(BadRequest, vhm1.manage_addToContainer, self.root)
-        self.assertRaises( BadRequest
-                         , manage_addVirtualHostMonster
-                         , self.root
-                         )
+        self.assertRaises(BadRequest, manage_addVirtualHostMonster, self.root)
 
     def test_add_addToContainer(self):
         from ZPublisher.BeforeTraverse import queryBeforeTraverse
@@ -266,4 +282,3 @@ class VHMAddingTests(unittest.TestCase):
         self.assertTrue(VirtualHostMonster.id in self.root.objectIds())
         hook = queryBeforeTraverse(self.root, VirtualHostMonster.meta_type)
         self.assertTrue(hook)
-
