@@ -13,30 +13,41 @@
 """Support for (functional) doc tests
 """
 
-import base64
 import doctest
+import email.parser
 import re
 import sys
 import warnings
+from doctest import ELLIPSIS
+from doctest import NORMALIZE_WHITESPACE
+from doctest import REPORT_NDIFF
+from functools import partial
+from io import BytesIO
+
+from six import text_type
 
 import transaction
-
-from Testing.ZopeTestCase import ZopeTestCase
-from Testing.ZopeTestCase import FunctionalTestCase
 from Testing.ZopeTestCase import Functional
+from Testing.ZopeTestCase import FunctionalTestCase
+from Testing.ZopeTestCase import ZopeTestCase
 from Testing.ZopeTestCase import folder_name
+from Testing.ZopeTestCase import standard_permissions
 from Testing.ZopeTestCase import user_name
 from Testing.ZopeTestCase import user_password
 from Testing.ZopeTestCase import user_role
-from Testing.ZopeTestCase import standard_permissions
-from Testing.ZopeTestCase.sandbox import AppZapper
 from Testing.ZopeTestCase.functional import ResponseWrapper
 from Testing.ZopeTestCase.functional import savestate
+from Testing.ZopeTestCase.sandbox import AppZapper
+from ZPublisher.httpexceptions import HTTPExceptionHandler
+from ZPublisher.utils import basic_auth_encode
 
 
-class HTTPHeaderOutput:
+if sys.version_info >= (3, ):
+    basestring = str
+
+
+class HTTPHeaderOutput(object):
 
-    # zope.interface.implements(zope.server.interfaces.IHeaderOutput)
     status = '200'
     reason = 'OK'
 
@@ -79,22 +90,27 @@ class DocResponseWrapper(ResponseWrapper):
     """Response Wrapper for use in doctests
     """
 
-    def __init__(self, response, outstream, path, header_output):
-        ResponseWrapper.__init__(self, response, outstream, path)
+    def __init__(self, response, outstream, path, header_output,
+                 wsgi_result=(), wsgi_headers=''):
+        ResponseWrapper.__init__(self, response, outstream, path,
+                                 wsgi_result, wsgi_headers)
         self.header_output = header_output
 
     def __str__(self):
-        body = self.getBody()
+        body = self._decode(self.getBody())
         if body:
             return "%s\n\n%s" % (self.header_output, body)
         return "%s\n" % (self.header_output)
 
 
-headerre = re.compile('(\S+): (.+)$')
+basicre = re.compile(r'Basic (.+)?:(.+)?$')
+headerre = re.compile(r'(\S+): (.+)$')
+
+
 def split_header(header):
     return headerre.match(header).group(1, 2)
 
-basicre = re.compile('Basic (.+)?:(.+)?$')
+
 def auth_header(header):
     match = basicre.match(header)
     if match:
@@ -103,14 +119,14 @@ def auth_header(header):
             u = ''
         if p is None:
             p = ''
-        auth = base64.encodestring('%s:%s' % (u, p))
-        return 'Basic %s' % auth[:-1]
+        return basic_auth_encode(u, p)
     return header
 
 
 def getRootFolder():
     return AppZapper().app()
 
+
 def sync():
     getRootFolder()._p_jar.sync()
 
@@ -121,11 +137,10 @@ def http(request_string, handle_errors=True):
 
     This is used for HTTP doc tests.
     """
-    import urllib
-    import rfc822
-    from cStringIO import StringIO
-    from ZPublisher.Response import Response
-    from ZPublisher.Publish import publish_module
+    from six.moves.urllib.parse import unquote
+    from ZPublisher.HTTPRequest import WSGIRequest as Request
+    from ZPublisher.HTTPResponse import WSGIResponse
+    from ZPublisher.WSGIPublisher import publish_module
 
     # Commit work done by previous python code.
     transaction.commit()
@@ -134,19 +149,18 @@ def http(request_string, handle_errors=True):
     request_string = request_string.lstrip()
 
     # Split off and parse the command line
-    l = request_string.find('\n')
-    command_line = request_string[:l].rstrip()
-    request_string = request_string[l+1:]
+    newline = request_string.find('\n')
+    command_line = request_string[:newline].rstrip()
+    request_string = request_string[newline + 1:]
     method, path, protocol = command_line.split()
-    path = urllib.unquote(path)
+    path = unquote(path)
 
-    instream = StringIO(request_string)
-
-    env = {"HTTP_HOST": 'localhost',
-           "HTTP_REFERER": 'localhost',
-           "REQUEST_METHOD": method,
-           "SERVER_PROTOCOL": protocol,
-           }
+    env = {
+        'HTTP_HOST': 'localhost',
+        'HTTP_REFERER': 'localhost',
+        'REQUEST_METHOD': method,
+        'SERVER_PROTOCOL': protocol,
+    }
 
     p = path.split('?', 1)
     if len(p) == 1:
@@ -154,18 +168,23 @@ def http(request_string, handle_errors=True):
     elif len(p) == 2:
         [env['PATH_INFO'], env['QUERY_STRING']] = p
     else:
-        raise TypeError, ''
+        raise TypeError('')
 
     header_output = HTTPHeaderOutput(
-        protocol, ('x-content-type-warning', 'x-powered-by',
-                   'bobo-exception-type', 'bobo-exception-file',
-                   'bobo-exception-value', 'bobo-exception-line'))
+        protocol, ('x-content-type-warning', 'x-powered-by'))
 
-    headers = [split_header(header)
-               for header in rfc822.Message(instream).headers]
+    # With a HeaderParser the payload is always a string, Parser would create a
+    # list of messages for multipart messages.
+    parser = email.parser.HeaderParser()
+    msg = parser.parsestr(request_string)
+    headers = msg.items()
+    body = msg.get_payload()
+
+    if isinstance(body, text_type):
+        body = body.encode('utf-8')
 
     # Store request body without headers
-    instream = StringIO(instream.read())
+    instream = BytesIO(body)
 
     for name, value in headers:
         name = ('_'.join(name.upper().split('-')))
@@ -173,18 +192,34 @@ def http(request_string, handle_errors=True):
             name = 'HTTP_' + name
         env[name] = value.rstrip()
 
-    if env.has_key('HTTP_AUTHORIZATION'):
+    if 'HTTP_AUTHORIZATION' in env:
         env['HTTP_AUTHORIZATION'] = auth_header(env['HTTP_AUTHORIZATION'])
 
-    outstream = StringIO()
-    response = Response(stdout=outstream, stderr=sys.stderr)
+    if not handle_errors:
+        # Tell the publisher to skip exception views
+        env['x-wsgiorg.throw_errors'] = True
+
+    outstream = BytesIO()
+    response = WSGIResponse(stdout=outstream, stderr=sys.stderr)
+    request = Request(instream, env, response)
+
+    env['wsgi.input'] = instream
+    wsgi_headers = BytesIO()
+
+    def start_response(status, headers):
+        wsgi_headers.write(
+            b'HTTP/1.1 ' + status.encode('ascii') + b'\r\n')
+        headers = b'\r\n'.join([
+            (k + ': ' + v).encode('ascii') for k, v in headers])
+        wsgi_headers.write(headers)
+        wsgi_headers.write(b'\r\n\r\n')
+
+    publish = partial(publish_module, _request=request, _response=response)
+    if handle_errors:
+        publish = HTTPExceptionHandler(publish)
+
+    wsgi_result = publish(env, start_response)
 
-    publish_module('Zope2',
-                   response=response,
-                   stdin=instream,
-                   environ=env,
-                   debug=not handle_errors,
-                  )
     header_output.setResponseStatus(response.getStatus(), response.errmsg)
     header_output.setResponseHeaders(response.headers)
     header_output.headersl.extend(response._cookie_list())
@@ -192,10 +227,11 @@ def http(request_string, handle_errors=True):
 
     sync()
 
-    return DocResponseWrapper(response, outstream, path, header_output)
+    return DocResponseWrapper(
+        response, outstream, path, header_output, wsgi_result, wsgi_headers)
 
 
-class ZopeSuiteFactory:
+class ZopeSuiteFactory(object):
 
     def __init__(self, *args, **kw):
         self._args = args
@@ -247,6 +283,7 @@ class ZopeSuiteFactory:
         test_instance = test_class()
 
         kwsetUp = self._kw.get('setUp')
+
         def setUp(test):
             test_instance.setUp()
             test.globs['test'] = test
@@ -265,6 +302,7 @@ class ZopeSuiteFactory:
         self._kw['setUp'] = setUp
 
         kwtearDown = self._kw.get('tearDown')
+
         def tearDown(test):
             if kwtearDown is not None:
                 kwtearDown(test_instance)
@@ -274,8 +312,7 @@ class ZopeSuiteFactory:
 
     def setup_optionflags(self):
         if 'optionflags' not in self._kw:
-            self._kw['optionflags'] = (doctest.ELLIPSIS
-                                       | doctest.NORMALIZE_WHITESPACE)
+            self._kw['optionflags'] = (ELLIPSIS | NORMALIZE_WHITESPACE)
 
 
 class FunctionalSuiteFactory(ZopeSuiteFactory):
@@ -286,7 +323,7 @@ class FunctionalSuiteFactory(ZopeSuiteFactory):
         globs['http'] = http
         globs['getRootFolder'] = getRootFolder
         globs['sync'] = sync
-        globs['user_auth'] = base64.encodestring('%s:%s' % (user_name, user_password))
+        globs['user_auth'] = basic_auth_encode(user_name, user_password)
 
     def setup_test_class(self):
         test_class = self._kw.get('test_class', FunctionalTestCase)
@@ -298,7 +335,7 @@ class FunctionalSuiteFactory(ZopeSuiteFactory):
             warnings.warn(("The test_class you are using doesn't "
                            "subclass from ZopeTestCase.Functional. "
                            "Please fix that."), UserWarning, 4)
-            if not 'Functional' in name:
+            if 'Functional' not in name:
                 name = 'Functional%s' % name
             test_class = type(name, (Functional, test_class), {})
 
@@ -307,24 +344,26 @@ class FunctionalSuiteFactory(ZopeSuiteFactory):
 
     def setup_optionflags(self):
         if 'optionflags' not in self._kw:
-            self._kw['optionflags'] = (doctest.ELLIPSIS
-                                       | doctest.REPORT_NDIFF
-                                       | doctest.NORMALIZE_WHITESPACE)
+            self._kw['optionflags'] = (
+                ELLIPSIS | REPORT_NDIFF | NORMALIZE_WHITESPACE)
 
 
 def ZopeDocTestSuite(module=None, **kw):
     module = doctest._normalize_module(module)
     return ZopeSuiteFactory(module, **kw).doctestsuite()
 
+
 def ZopeDocFileSuite(*paths, **kw):
     if kw.get('module_relative', True):
         kw['package'] = doctest._normalize_module(kw.get('package'))
     return ZopeSuiteFactory(*paths, **kw).docfilesuite()
 
+
 def FunctionalDocTestSuite(module=None, **kw):
     module = doctest._normalize_module(module)
     return FunctionalSuiteFactory(module, **kw).doctestsuite()
 
+
 def FunctionalDocFileSuite(*paths, **kw):
     if kw.get('module_relative', True):
         kw['package'] = doctest._normalize_module(kw.get('package'))
@@ -336,4 +375,4 @@ __all__ = [
     'ZopeDocFileSuite',
     'FunctionalDocTestSuite',
     'FunctionalDocFileSuite',
-    ]
+]
