@@ -1,5 +1,7 @@
 # Import all patches.
-import publishing
+from . import persistence
+from . import publishing
+
 
 # Have the patches been applied yet?
 _patched = False
@@ -11,4 +13,5 @@ def apply_patches():
         return
     _patched = True
 
+    persistence.apply_patches()
     publishing.apply_patches()
