@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Foundation and Contributors.
 #
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
@@ -11,63 +11,63 @@
 #
 ##############################################################################
 """WebDAV support - resource objects.
-
-$Id: Resource.py 100339 2009-05-24 17:24:00Z hannosch $
 """
 
 import mimetypes
-import sys
-import warnings
 import re
-from urllib import unquote
+import sys
+
+from six.moves.urllib.parse import unquote
 
-from AccessControl import getSecurityManager
 from AccessControl import ClassSecurityInfo
+from AccessControl import getSecurityManager
+from AccessControl.class_init import InitializeClass
 from AccessControl.Permissions import delete_objects
 from AccessControl.Permissions import manage_properties
-from AccessControl.Permissions import view as View
+from AccessControl.Permissions import view
+from AccessControl.Permissions import webdav_access
 from AccessControl.Permissions import webdav_lock_items
 from AccessControl.Permissions import webdav_unlock_items
-from AccessControl.Permissions import webdav_access
 from Acquisition import aq_base
 from Acquisition import aq_inner
 from Acquisition import aq_parent
-from App.class_init import InitializeClass
+from App.Common import rfc1123_date
 from ExtensionClass import Base
 from OFS.event import ObjectClonedEvent
 from OFS.event import ObjectWillBeMovedEvent
+from OFS.interfaces import IWriteLock
+from OFS.Lockable import LockableItem
+from OFS.Lockable import wl_isLockable
+from OFS.Lockable import wl_isLocked
 from OFS.subscribers import compatibilityCall
+from webdav import enable_ms_public_header
+from webdav.common import Conflict
+from webdav.common import IfParser
+from webdav.common import Locked
+from webdav.common import PreconditionFailed
+from webdav.common import absattr
+from webdav.common import isDavCollection
+from webdav.common import tokenFinder
+from webdav.common import urlbase
+from webdav.common import urlfix
+from webdav.interfaces import IDAVResource
 from zExceptions import BadRequest
 from zExceptions import Forbidden
 from zExceptions import MethodNotAllowed
 from zExceptions import NotFound
 from zExceptions import Unauthorized
-from ZPublisher.HTTPRangeSupport import HTTPRangeInterface
-
-from zope.interface import implements
+from zope.container.contained import notifyContainerModified
 from zope.event import notify
+from zope.interface import implementer
 from zope.lifecycleevent import ObjectCopiedEvent
 from zope.lifecycleevent import ObjectMovedEvent
-from zope.container.contained import notifyContainerModified
+from ZPublisher.HTTPRangeSupport import HTTPRangeInterface
 
-from webdav.Lockable import LockableItem
-from webdav.Lockable import wl_isLockable
-from webdav.Lockable import wl_isLocked
-from webdav.common import absattr
-from webdav.common import Conflict
-from webdav.common import IfParser
-from webdav.common import isDavCollection
-from webdav.common import Locked
-from webdav.common import PreconditionFailed
-from webdav.common import rfc1123_date
-from webdav.common import tokenFinder
-from webdav.common import urlbase
-from webdav.common import urlfix
-from webdav.interfaces import IDAVResource
-from webdav.interfaces import IWriteLock
 
 ms_dav_agent = re.compile("Microsoft.*Internet Publishing.*")
 
+
+@implementer(IDAVResource)
 class Resource(Base, LockableItem):
 
     """The Resource mixin class provides basic WebDAV support for
@@ -76,14 +76,12 @@ class Resource(Base, LockableItem):
     such as PUT should be overridden to ensure correct behavior in
     the context of the object type."""
 
-    implements(IDAVResource)
+    __dav_resource__ = 1
 
-    __dav_resource__=1
-
-    __http_methods__=('GET', 'HEAD', 'POST', 'PUT', 'DELETE', 'OPTIONS',
-                      'TRACE', 'PROPFIND', 'PROPPATCH', 'MKCOL', 'COPY',
-                      'MOVE', 'LOCK', 'UNLOCK',
-                      )
+    __http_methods__ = ('GET', 'HEAD', 'POST', 'PUT', 'DELETE', 'OPTIONS',
+                        'TRACE', 'PROPFIND', 'PROPPATCH', 'MKCOL', 'COPY',
+                        'MOVE', 'LOCK', 'UNLOCK',
+                        )
 
     security = ClassSecurityInfo()
     security.setPermissionDefault(webdav_access, ('Authenticated', 'Manager'))
@@ -96,9 +94,11 @@ class Resource(Base, LockableItem):
         # want to write duplicate headers (since ZS writes Date
         # and Connection itself).
         if not hasattr(response, '_server_version'):
-            response.setHeader('Connection', 'close')
             response.setHeader('Date', rfc1123_date(), 1)
 
+        # Initialize ETag header
+        self.http__etag()
+
         # HTTP Range support
         if HTTPRangeInterface.providedBy(self):
             response.setHeader('Accept-Ranges', 'bytes')
@@ -106,22 +106,26 @@ class Resource(Base, LockableItem):
             response.setHeader('Accept-Ranges', 'none')
 
     def dav__validate(self, object, methodname, REQUEST):
-        msg='<strong>You are not authorized to access this resource.</strong>'
-        method=None
+        msg = ('<strong>You are not authorized '
+               'to access this resource.</strong>')
+        method = None
         if hasattr(object, methodname):
-            method=getattr(object, methodname)
+            method = getattr(object, methodname)
         else:
-            try:    method=object.aq_acquire(methodname)
-            except: method=None
+            try:
+                method = object.aq_acquire(methodname)
+            except Exception:
+                method = None
 
         if method is not None:
             try:
                 return getSecurityManager().validate(None, object,
                                                      methodname,
                                                      method)
-            except: pass
+            except Exception:
+                pass
 
-        raise Unauthorized, msg
+        raise Unauthorized(msg)
 
     def dav__simpleifhandler(self, request, response, method='PUT',
                              col=0, url=None, refresh=0):
@@ -143,64 +147,74 @@ class Resource(Base, LockableItem):
         # Since we're a simple if handler, and since some clients don't
         # pass in the port information in the resource part of an If
         # header, we're only going to worry about if the paths compare
-        if url is None: url = urlfix(request['URL'], method)
-        url = urlbase(url)              # Gets just the path information
+        if url is None:
+            url = urlfix(request['URL'], method)
+        url = urlbase(url)  # Gets just the path information
 
         # if 'col' is passed in, an operation is happening on a submember
         # of a collection, while the Lock may be on the parent.  Lob off
         # the final part of the URL  (ie '/a/b/foo.html' becomes '/a/b/')
-        if col: url = url[:url.rfind('/')+1]
+        if col:
+            url = url[:url.rfind('/') + 1]
 
-        havetag = lambda x, self=self: self.wl_hasLock(x)
-        found = 0; resourcetagged = 0
+        found = 0
+        resourcetagged = 0
         taglist = IfParser(ifhdr)
         for tag in taglist:
 
             if not tag.resource:
                 # There's no resource (url) with this tag
-                tag_list = map(tokenFinder, tag.list)
-                wehave = filter(havetag, tag_list)
+                tag_list = [tokenFinder(x) for x in tag.list]
+                wehave = [t for t in tag_list if self.wl_hasLock(t)]
 
-                if not wehave: continue
-                if tag.NOTTED: continue
+                if not wehave:
+                    continue
+                if tag.NOTTED:
+                    continue
                 if refresh:
-                    for token in wehave: self.wl_getLock(token).refresh()
+                    for token in wehave:
+                        self.wl_getLock(token).refresh()
                 resourcetagged = 1
-                found = 1; break
-            elif urlbase(tag.resource) == url:
+                found = 1
+                break
+            elif unquote(urlbase(tag.resource)) == unquote(url):
                 resourcetagged = 1
-                tag_list = map(tokenFinder, tag.list)
-                wehave = filter(havetag, tag_list)
+                tag_list = [tokenFinder(x) for x in tag.list]
+                wehave = [t for t in tag_list if self.wl_hasLock(t)]
 
-                if not wehave: continue
-                if tag.NOTTED: continue
+                if not wehave:
+                    continue
+                if tag.NOTTED:
+                    continue
                 if refresh:
-                    for token in wehave: self.wl_getLock(token).refresh()
-                found = 1; break
+                    for token in wehave:
+                        self.wl_getLock(token).refresh()
+                found = 1
+                break
 
         if resourcetagged and (not found):
-            raise PreconditionFailed, 'Condition failed.'
+            raise PreconditionFailed('Condition failed.')
         elif resourcetagged and found:
             return 1
         else:
             return 0
 
     # WebDAV class 1 support
-    security.declareProtected(View, 'HEAD')
+    @security.protected(view)
     def HEAD(self, REQUEST, RESPONSE):
         """Retrieve resource information without a response body."""
         self.dav__init(REQUEST, RESPONSE)
 
-        content_type=None
+        content_type = None
         if hasattr(self, 'content_type'):
-            content_type=absattr(self.content_type)
+            content_type = absattr(self.content_type)
         if content_type is None:
-            url=urlfix(REQUEST['URL'], 'HEAD')
-            name=unquote(filter(None, url.split( '/')[-1]))
-            content_type, encoding=mimetypes.guess_type(name)
+            url = urlfix(REQUEST['URL'], 'HEAD')
+            name = unquote([_f for _f in url.split('/') if _f][-1])
+            content_type, encoding = mimetypes.guess_type(name)
         if content_type is None:
             if hasattr(self, 'default_content_type'):
-                content_type=absattr(self.default_content_type)
+                content_type = absattr(self.default_content_type)
         if content_type is None:
             content_type = 'application/octet-stream'
         RESPONSE.setHeader('Content-Type', content_type.lower())
@@ -208,7 +222,7 @@ class Resource(Base, LockableItem):
         if hasattr(aq_base(self), 'get_size'):
             RESPONSE.setHeader('Content-Length', absattr(self.get_size))
         if hasattr(self, '_p_mtime'):
-            mtime=rfc1123_date(self._p_mtime)
+            mtime = rfc1123_date(self._p_mtime)
             RESPONSE.setHeader('Last-Modified', mtime)
         if hasattr(aq_base(self), 'http__etag'):
             etag = self.http__etag(readonly=1)
@@ -224,12 +238,11 @@ class Resource(Base, LockableItem):
         object-specific implementation. By default, PUT requests
         fail with a 405 (Method Not Allowed)."""
         self.dav__init(REQUEST, RESPONSE)
-        raise MethodNotAllowed, 'Method not supported for this resource.'
+        raise MethodNotAllowed('Method not supported for this resource.')
 
-    security.declarePublic('OPTIONS')
+    @security.public
     def OPTIONS(self, REQUEST, RESPONSE):
         """Retrieve communication options."""
-        import webdav
         self.dav__init(REQUEST, RESPONSE)
         RESPONSE.setHeader('Allow', ', '.join(self.__http_methods__))
         RESPONSE.setHeader('Content-Length', 0)
@@ -238,15 +251,13 @@ class Resource(Base, LockableItem):
         # Microsoft Web Folders compatibility, only enabled if
         # User-Agent matches.
         if ms_dav_agent.match(REQUEST.get_header('User-Agent', '')):
-            if webdav.enable_ms_public_header:
+            if enable_ms_public_header:
                 RESPONSE.setHeader('Public', ', '.join(self.__http_methods__))
-            if webdav.enable_ms_author_via:
-                RESPONSE.setHeader('MS-Author-Via', 'DAV')
 
         RESPONSE.setStatus(200)
         return RESPONSE
 
-    security.declarePublic('TRACE')
+    @security.public
     def TRACE(self, REQUEST, RESPONSE):
         """Return the HTTP message received back to the client as the
         entity-body of a 200 (OK) response. This will often usually
@@ -255,16 +266,16 @@ class Resource(Base, LockableItem):
         is not often possible to reproduce the HTTP request verbatim
         from within the Zope environment."""
         self.dav__init(REQUEST, RESPONSE)
-        raise MethodNotAllowed, 'Method not supported for this resource.'
+        raise MethodNotAllowed('Method not supported for this resource.')
 
-    security.declareProtected(delete_objects, 'DELETE')
+    @security.protected(delete_objects)
     def DELETE(self, REQUEST, RESPONSE):
         """Delete a resource. For non-collection resources, DELETE may
         return either 200 or 204 (No Content) to indicate success."""
         self.dav__init(REQUEST, RESPONSE)
         ifhdr = REQUEST.get_header('If', '')
         url = urlfix(REQUEST['URL'], 'DELETE')
-        name = unquote(filter(None, url.split( '/')[-1]))
+        name = unquote([_f for _f in url.split('/') if _f][-1])
         parent = aq_parent(aq_inner(self))
         # Lock checking
         if wl_isLocked(self):
@@ -273,20 +284,19 @@ class Resource(Base, LockableItem):
             else:
                 # We're locked, and no if header was passed in, so
                 # the client doesn't own a lock.
-                raise Locked, 'Resource is locked.'
+                raise Locked('Resource is locked.')
         elif IWriteLock.providedBy(parent) and parent.wl_isLocked():
             if ifhdr:
                 parent.dav__simpleifhandler(REQUEST, RESPONSE, 'DELETE', col=1)
             else:
                 # Our parent is locked, and no If header was passed in.
                 # When a parent is locked, members cannot be removed
-                raise PreconditionFailed, 'Resource is locked, and no '\
-                      'condition was passed in.'
+                raise Locked('Parent of this resource is locked.')
         # Either we're not locked, or a succesful lock token was submitted
         # so we can delete the lock now.
         # ajung: Fix for Collector # 2196
 
-        if parent.manage_delObjects([name],REQUEST=None)  is None:
+        if parent.manage_delObjects([name], REQUEST=None) is None:
             RESPONSE.setStatus(204)
         else:
 
@@ -294,7 +304,7 @@ class Resource(Base, LockableItem):
 
         return RESPONSE
 
-    security.declareProtected(webdav_access, 'PROPFIND')
+    @security.protected(webdav_access)
     def PROPFIND(self, REQUEST, RESPONSE):
         """Retrieve properties defined on the resource."""
         from webdav.davcmds import PropFind
@@ -302,32 +312,32 @@ class Resource(Base, LockableItem):
         cmd = PropFind(REQUEST)
         result = cmd.apply(self)
         # work around MSIE DAV bug for creation and modified date
-        if (REQUEST.get_header('User-Agent') ==
-            'Microsoft Data Access Internet Publishing Provider DAV 1.1'):
+        if REQUEST.get_header('User-Agent') == \
+           'Microsoft Data Access Internet Publishing Provider DAV 1.1':
             result = result.replace('<n:getlastmodified xmlns:n="DAV:">',
-                                    '<n:getlastmodified xmlns:n="DAV:" xmlns:b="urn:uuid:c2f41010-65b3-11d1-a29f-00aa00c14882/" b:dt="dateTime.rfc1123">')
+                                    '<n:getlastmodified xmlns:n="DAV:" xmlns:b="urn:uuid:c2f41010-65b3-11d1-a29f-00aa00c14882/" b:dt="dateTime.rfc1123">')  # NOQA
             result = result.replace('<n:creationdate xmlns:n="DAV:">',
-                                    '<n:creationdate xmlns:n="DAV:" xmlns:b="urn:uuid:c2f41010-65b3-11d1-a29f-00aa00c14882/" b:dt="dateTime.tz">')
+                                    '<n:creationdate xmlns:n="DAV:" xmlns:b="urn:uuid:c2f41010-65b3-11d1-a29f-00aa00c14882/" b:dt="dateTime.tz">')  # NOQA
         RESPONSE.setStatus(207)
         RESPONSE.setHeader('Content-Type', 'text/xml; charset="utf-8"')
         RESPONSE.setBody(result)
         return RESPONSE
 
-    security.declareProtected(manage_properties, 'PROPPATCH')
+    @security.protected(manage_properties)
     def PROPPATCH(self, REQUEST, RESPONSE):
         """Set and/or remove properties defined on the resource."""
         from webdav.davcmds import PropPatch
         self.dav__init(REQUEST, RESPONSE)
         if not hasattr(aq_base(self), 'propertysheets'):
-            raise MethodNotAllowed, (
-                  'Method not supported for this resource.')
+            raise MethodNotAllowed(
+                'Method not supported for this resource.')
         # Lock checking
         ifhdr = REQUEST.get_header('If', '')
         if wl_isLocked(self):
             if ifhdr:
                 self.dav__simpleifhandler(REQUEST, RESPONSE, 'PROPPATCH')
             else:
-                raise Locked, 'Resource is locked.'
+                raise Locked('Resource is locked.')
 
         cmd = PropPatch(REQUEST)
         result = cmd.apply(self)
@@ -340,9 +350,9 @@ class Resource(Base, LockableItem):
         """Create a new collection resource. If called on an existing
         resource, MKCOL must fail with 405 (Method Not Allowed)."""
         self.dav__init(REQUEST, RESPONSE)
-        raise MethodNotAllowed, 'The resource already exists.'
+        raise MethodNotAllowed('The resource already exists.')
 
-    security.declarePublic('COPY')
+    @security.public
     def COPY(self, REQUEST, RESPONSE):
         """Create a duplicate of the source resource whose state
         and behavior match that of the source resource as closely
@@ -352,51 +362,53 @@ class Resource(Base, LockableItem):
         self.dav__init(REQUEST, RESPONSE)
         if not hasattr(aq_base(self), 'cb_isCopyable') or \
            not self.cb_isCopyable():
-            raise MethodNotAllowed, 'This object may not be copied.'
+            raise MethodNotAllowed('This object may not be copied.')
 
-        depth=REQUEST.get_header('Depth', 'infinity')
-        if not depth in ('0', 'infinity'):
-            raise BadRequest, 'Invalid Depth header.'
+        depth = REQUEST.get_header('Depth', 'infinity')
+        if depth not in ('0', 'infinity'):
+            raise BadRequest('Invalid Depth header.')
 
-        dest=REQUEST.get_header('Destination', '')
-        while dest and dest[-1]=='/':
-            dest=dest[:-1]
+        dest = REQUEST.get_header('Destination', '')
+        while dest and dest[-1] == '/':
+            dest = dest[:-1]
         if not dest:
-            raise BadRequest, 'Invalid Destination header.'
+            raise BadRequest('Invalid Destination header.')
 
-        try: path = REQUEST.physicalPathFromURL(dest)
+        try:
+            path = REQUEST.physicalPathFromURL(dest)
         except ValueError:
-            raise BadRequest, 'Invalid Destination header'
+            raise BadRequest('Invalid Destination header')
 
         name = path.pop()
 
-        oflag=REQUEST.get_header('Overwrite', 'F').upper()
-        if not oflag in ('T', 'F'):
-            raise BadRequest, 'Invalid Overwrite header.'
+        oflag = REQUEST.get_header('Overwrite', 'F').upper()
+        if oflag not in ('T', 'F'):
+            raise BadRequest('Invalid Overwrite header.')
 
-        try: parent=self.restrictedTraverse(path)
+        try:
+            parent = self.restrictedTraverse(path)
         except ValueError:
-            raise Conflict, 'Attempt to copy to an unknown namespace.'
+            raise Conflict('Attempt to copy to an unknown namespace.')
         except NotFound:
-            raise Conflict, 'Object ancestors must already exist.'
-        except:
-            t, v, tb=sys.exc_info()
-            raise t, v
+            raise Conflict('Object ancestors must already exist.')
+        except Exception:
+            raise
+
         if hasattr(parent, '__null_resource__'):
-            raise Conflict, 'Object ancestors must already exist.'
-        existing=hasattr(aq_base(parent), name)
-        if existing and oflag=='F':
-            raise PreconditionFailed, 'Destination resource exists.'
+            raise Conflict('Object ancestors must already exist.')
+        existing = hasattr(aq_base(parent), name)
+        if existing and oflag == 'F':
+            raise PreconditionFailed('Destination resource exists.')
         try:
             parent._checkId(name, allow_dup=1)
-        except:
-            raise Forbidden, sys.exc_info()[1]
+        except Exception:
+            raise Forbidden(sys.exc_info()[1])
         try:
             parent._verifyObjectPaste(self)
         except Unauthorized:
             raise
-        except:
-            raise Forbidden, sys.exc_info()[1]
+        except Exception:
+            raise Forbidden(sys.exc_info()[1])
 
         # Now check locks.  The If header on a copy only cares about the
         # lock on the destination, so we need to check out the destinations
@@ -410,28 +422,28 @@ class Resource(Base, LockableItem):
                     itrue = destob.dav__simpleifhandler(
                         REQUEST, RESPONSE, 'COPY', refresh=1)
                     if not itrue:
-                        raise PreconditonFailed
+                        raise PreconditionFailed()
                 else:
-                    raise Locked, 'Destination is locked.'
+                    raise Locked('Destination is locked.')
         elif IWriteLock.providedBy(parent) and parent.wl_isLocked():
             if ifhdr:
                 parent.dav__simpleifhandler(REQUEST, RESPONSE, 'COPY',
                                             refresh=1)
             else:
-                raise Locked, 'Destination is locked.'
+                raise Locked('Destination is locked.')
 
         self._notifyOfCopyTo(parent, op=0)
         ob = self._getCopy(parent)
         ob._setId(name)
 
-        if depth=='0' and isDavCollection(ob):
+        if depth == '0' and isDavCollection(ob):
             for id in ob.objectIds():
                 ob._delObject(id)
 
         notify(ObjectCopiedEvent(ob, self))
 
         if existing:
-            object=getattr(parent, name)
+            object = getattr(parent, name)
             self.dav__validate(object, 'DELETE', REQUEST)
             parent._delObject(name)
 
@@ -452,7 +464,7 @@ class Resource(Base, LockableItem):
         RESPONSE.setBody('')
         return RESPONSE
 
-    security.declarePublic('MOVE')
+    @security.public
     def MOVE(self, REQUEST, RESPONSE):
         """Move a resource to a new location. Though we may later try to
         make a move appear seamless across namespaces (e.g. from Zope
@@ -462,42 +474,45 @@ class Resource(Base, LockableItem):
         self.dav__validate(self, 'DELETE', REQUEST)
         if not hasattr(aq_base(self), 'cb_isMoveable') or \
            not self.cb_isMoveable():
-            raise MethodNotAllowed, 'This object may not be moved.'
+            raise MethodNotAllowed('This object may not be moved.')
 
-        dest=REQUEST.get_header('Destination', '')
+        dest = REQUEST.get_header('Destination', '')
 
-        try: path = REQUEST.physicalPathFromURL(dest)
+        try:
+            path = REQUEST.physicalPathFromURL(dest)
         except ValueError:
-            raise BadRequest, 'No destination given'
+            raise BadRequest('No destination given')
 
-        flag=REQUEST.get_header('Overwrite', 'F')
-        flag=flag.upper()
+        flag = REQUEST.get_header('Overwrite', 'F')
+        flag = flag.upper()
 
         name = path.pop()
         parent_path = '/'.join(path)
 
-        try: parent = self.restrictedTraverse(path)
+        try:
+            parent = self.restrictedTraverse(path)
         except ValueError:
-            raise Conflict, 'Attempt to move to an unknown namespace.'
+            raise Conflict('Attempt to move to an unknown namespace.')
         except 'Not Found':
-            raise Conflict, 'The resource %s must exist.' % parent_path
-        except:
+            raise Conflict('The resource %s must exist.' % parent_path)
+        except Exception:
             raise
+
         if hasattr(parent, '__null_resource__'):
-            raise Conflict, 'The resource %s must exist.' % parent_path
-        existing=hasattr(aq_base(parent), name)
-        if existing and flag=='F':
-            raise PreconditionFailed, 'Resource %s exists.' % dest
+            raise Conflict('The resource %s must exist.' % parent_path)
+        existing = hasattr(aq_base(parent), name)
+        if existing and flag == 'F':
+            raise PreconditionFailed('Resource %s exists.' % dest)
         try:
             parent._checkId(name, allow_dup=1)
-        except:
-            raise Forbidden, sys.exc_info()[1]
+        except Exception:
+            raise Forbidden(sys.exc_info()[1])
         try:
             parent._verifyObjectPaste(self)
         except Unauthorized:
             raise
-        except:
-            raise Forbidden, sys.exc_info()[1]
+        except Exception:
+            raise Forbidden(sys.exc_info()[1])
 
         # Now check locks.  Since we're affecting the resource that we're
         # moving as well as the destination, we have to check both.
@@ -512,7 +527,7 @@ class Resource(Base, LockableItem):
                     if not itrue:
                         raise PreconditionFailed
                 else:
-                    raise Locked, 'Destination is locked.'
+                    raise Locked('Destination is locked.')
         elif IWriteLock.providedBy(parent) and parent.wl_isLocked():
             # There's no existing object in the destination folder, so
             # we need to check the folders locks since we're changing its
@@ -521,16 +536,16 @@ class Resource(Base, LockableItem):
                 itrue = parent.dav__simpleifhandler(REQUEST, RESPONSE, 'MOVE',
                                                     col=1, url=dest, refresh=1)
                 if not itrue:
-                    raise PreconditionFailed, 'Condition failed.'
+                    raise PreconditionFailed('Condition failed.')
             else:
-                raise Locked, 'Destination is locked.'
+                raise Locked('Destination is locked.')
         if wl_isLocked(self):
             # Lastly, we check ourselves
             if ifhdr:
                 itrue = self.dav__simpleifhandler(REQUEST, RESPONSE, 'MOVE',
                                                   refresh=1)
                 if not itrue:
-                    raise PreconditionFailed, 'Condition failed.'
+                    raise PreconditionFailed('Condition failed.')
             else:
                 raise Locked('Source is locked and no condition was passed in')
 
@@ -552,7 +567,7 @@ class Resource(Base, LockableItem):
         orig_container._delObject(orig_id, suppress_events=True)
 
         if existing:
-            object=getattr(parent, name)
+            object = getattr(parent, name)
             self.dav__validate(object, 'DELETE', REQUEST)
             parent._delObject(name)
 
@@ -575,10 +590,9 @@ class Resource(Base, LockableItem):
         RESPONSE.setBody('')
         return RESPONSE
 
-
     # WebDAV Class 2, Lock and Unlock
 
-    security.declareProtected(webdav_lock_items, 'LOCK')
+    @security.protected(webdav_lock_items)
     def LOCK(self, REQUEST, RESPONSE):
         """Lock a resource"""
         from webdav.davcmds import Lock
@@ -616,7 +630,7 @@ class Resource(Base, LockableItem):
         else:
             # There's no body, so this likely to be a refresh request
             if not ifhdr:
-                raise PreconditionFailed, 'If Header Missing'
+                raise PreconditionFailed('If Header Missing')
             taglist = IfParser(ifhdr)
             found = 0
             for tag in taglist:
@@ -625,7 +639,7 @@ class Resource(Base, LockableItem):
                     if token and self.wl_hasLock(token):
                         lock = self.wl_getLock(token)
                         timeout = REQUEST.get_header('Timeout', 'Infinite')
-                        lock.setTimeout(timeout) # automatically refreshes
+                        lock.setTimeout(timeout)  # automatically refreshes
                         found = 1
 
                         RESPONSE.setStatus(200)
@@ -633,18 +647,18 @@ class Resource(Base, LockableItem):
                                            'text/xml; charset="utf-8"')
                         RESPONSE.setBody(lock.asXML())
                         break
-                if found: break
+                if found:
+                    break
             if not found:
-                RESPONSE.setStatus(412) # Precondition failed
+                RESPONSE.setStatus(412)  # Precondition failed
 
         return RESPONSE
 
-    security.declareProtected(webdav_unlock_items, 'UNLOCK')
+    @security.protected(webdav_unlock_items)
     def UNLOCK(self, REQUEST, RESPONSE):
         """Remove an existing lock on a resource."""
         from webdav.davcmds import Unlock
         self.dav__init(REQUEST, RESPONSE)
-        security = getSecurityManager()
         token = REQUEST.get_header('Lock-Token', '')
         url = REQUEST['URL']
         token = tokenFinder(token)
@@ -660,15 +674,25 @@ class Resource(Base, LockableItem):
             RESPONSE.setStatus(204)     # No Content response code
         return RESPONSE
 
-
-    security.declareProtected(webdav_access, 'manage_DAVget')
+    @security.protected(webdav_access)
     def manage_DAVget(self):
-        """Gets the document source"""
-        # The default implementation calls manage_FTPget
-        return self.manage_FTPget()
+        """Gets the document source or file data.
 
-    security.declareProtected(webdav_access, 'listDAVObjects')
+        This implementation is a last resort fallback. The subclass should
+        override this method to provide a more appropriate implementation.
+
+        Using PrincipiaSearchSource, if it exists. It is one of the few shared
+        interfaces still around in common Zope content objects.
+        """
+        if getattr(aq_base(self), 'PrincipiaSearchSource', None) is not None:
+            return self.PrincipiaSearchSource()
+
+        # If it doesn't exist, give up.
+        return ''
+
+    @security.protected(webdav_access)
     def listDAVObjects(self):
         return []
 
+
 InitializeClass(Resource)
